library angular2.test.router.router_link_spec;

import "package:angular2/testing_internal.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        xdescribe,
        describe,
        dispatchEvent,
        expect,
        iit,
        inject,
        beforeEachBindings,
        it,
        xit,
        TestComponentBuilder;
import "spies.dart" show SpyRouter, SpyLocation;
import "package:angular2/core.dart" show provide, Component, View;
import "package:angular2/src/core/debug.dart" show By;
import "package:angular2/router.dart"
    show
        Location,
        Router,
        RouteRegistry,
        RouterLink,
        RouterOutlet,
        Route,
        RouteParams,
        Instruction,
        ComponentInstruction;
import "package:angular2/src/core/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/router/instruction.dart"
    show ComponentInstruction_;

var dummyInstruction =
    new Instruction(new ComponentInstruction_("detail", [], null), null, {});
main() {
  describe("router-link directive", () {
    TestComponentBuilder tcb;
    beforeEachBindings(() => [
          provide(Location, useValue: makeDummyLocation()),
          provide(Router, useValue: makeDummyRouter())
        ]);
    beforeEach(inject([TestComponentBuilder], (tcBuilder) {
      tcb = tcBuilder;
    }));
    it(
        "should update a[href] attribute",
        inject([AsyncTestCompleter], (async) {
          tcb.createAsync(TestComponent).then((testComponent) {
            testComponent.detectChanges();
            var anchorElement =
                testComponent.debugElement.query(By.css("a")).nativeElement;
            expect(DOM.getAttribute(anchorElement, "href")).toEqual("/detail");
            async.done();
          });
        }));
    it(
        "should call router.navigate when a link is clicked",
        inject([AsyncTestCompleter, Router], (async, router) {
          tcb.createAsync(TestComponent).then((testComponent) {
            testComponent.detectChanges();
            // TODO: shouldn't this be just 'click' rather than '^click'?
            testComponent.debugElement
                .query(By.css("a"))
                .triggerEventHandler("click", null);
            expect(router.spy("navigateByInstruction"))
                .toHaveBeenCalledWith(dummyInstruction);
            async.done();
          });
        }));
  });
}

@Component(selector: "my-comp")
class MyComp {
  var name;
}

@Component(selector: "user-cmp")
@View(template: "hello {{user}}")
class UserCmp {
  String user;
  UserCmp(RouteParams params) {
    this.user = params.get("name");
  }
}

@Component(selector: "test-component")
@View(
    template: '''
    <div>
      <a [router-link]="[\'/Detail\']">detail view</a>
    </div>''',
    directives: const [RouterLink])
class TestComponent {}

makeDummyLocation() {
  var dl = new SpyLocation();
  dl.spy("normalizeAbsolutely").andCallFake((url) => url);
  return dl;
}

makeDummyRouter() {
  var dr = new SpyRouter();
  dr.spy("generate").andCallFake((routeParams) => dummyInstruction);
  dr.spy("isRouteActive").andCallFake((_) => false);
  dr.spy("navigateInstruction");
  return dr;
}
